(async () => {
  try {
    console.log("Starting tests")
    const { openBrowser, goto, textBox, into, write, click, toRightOf, below, link, press, image, waitFor, closeBrowser, screenshot } = require('taiko');
    const assert = require('assert');
    const globalTimeout = 60000

    const sso_username = process.env.SSO_USERNAME
    const sso_user_pw = process.env.SSO_USER_PW
    const installed_apps = process.env.INSTALLED_APPS.split(",")

    // Logs the user in with OIDC, uses `sso_username` and `sso_user_pw` consts
    async function sso_login_if_needed (button_text) {
      try {
        await console.log('Logging in with SSO...')
        await text(button_text).exists()
        await click(button_text)
        await write(sso_username, into(textBox('E-mail address')))
        await write(sso_user_pw, into(textBox('Password')))
        await click('Go!')
      } catch (error) {
        await console.error(error)
        console.log('Looks like we do not need to login, continuing')
      }
    }

    // Logs the user in with Nextcloud login interface, uses above consts
    async function login_if_needed() {
      try {
        const nextcloudAdminUsername = process.env.NEXTCLOUD_ADMIN_USERNAME
        const nextcloudAdminPassword = process.env.NEXTCLOUD_ADMIN_PASSWORD
        await console.log('Logging in without SSO...')
        await write(nextcloudAdminUsername, into(textBox('Username or email')))
        await write(nextcloudAdminPassword, into(textBox('Password')))
        await click('Log in')
      } catch (error) {
        await console.error(error)
        console.log('Looks like we do not need to login, continuing')
      }

    }

    setConfig( {
      observeTime: 0,
      // Navigation timeout value in milliseconds for navigation after performing openTab, goto, reload, goBack, goForward, click, write, clear, press and evaluate.
      navigationTimeout: globalTimeout,
      highlightOnAction: 'true'
    });

    await openBrowser({
      // Optimize chrome instances for docker an parallel runs, see:
      // https://docs.taiko.dev/taiko_in_docker/
      // https://docs.taiko.dev/frequently_asked_questions/#how-can-i-optimize-chrome-instances-for-parallel-runs%3F
      args: [
        "--disable-gpu",
        "--disable-dev-shm-usage",
        "--disable-setuid-sandbox",
        "--no-first-run",
        "--no-sandbox",
        "--no-zygote",
        "--start-maximized"
      ]
    });

    const nextcloudUrl = process.env.NEXTCLOUD_URL
    const onlyofficeUrl = process.env.ONLYOFFICE_URL
    console.log('• Onlyoffice')

    await goto(onlyofficeUrl + '/welcome')
    await waitFor(async () => (await text("Thank you for choosing ONLYOFFICE!").exists()))
    await waitFor(async () => (await text("Document Server is running").exists()))

    await goto(onlyofficeUrl + '/healthcheck')
    await waitFor(async () => (await text("true").exists()))

    console.log('• Nextcloud')
    await goto(nextcloudUrl)
    if (process.env.USE_SSO_LOGIN == "true") {
        await sso_login_if_needed('Log in with Stackspin')
    } else {
        login_if_needed()
    }
    await waitFor(async () => (await text("Recommended files").exists()), globalTimeout)
    // Close potential nextcloud first run wizard modal
    // https://github.com/nextcloud/firstrunwizard/issues/488
    // Unfortunately, we need to sleep a while since I haven't found a
    // good way that closes the modal *if* it pops up, since these
    // tests should also work on subsequent logins.
    await waitFor(5000)
    await press('Escape')

    // Test password app
    if (installed_apps.indexOf("passwords") != -1) {
        console.log('• Passwords application')
        await click('Passwords')
        await waitFor(async () => (await text("Trash").exists()), globalTimeout)
    }

    // Test if calendar app is enabled
    if (installed_apps.indexOf("calendar") != -1) {
        console.log('• Calendar application')
        await click('Calendar')
        await waitFor(async () => (await text("Settings & Import").exists()), globalTimeout)
    }

    console.log('• Nextcloud Onlyoffice integration')
    // Open document and type some text
    await click('Files')
    // Force page reload because of random empty pages for files app
    // https://open.greenhost.net/stackspin/nextcloud/-/issues/973
    await reload()

    await waitFor(async () => (await link({class:'new'}).isVisible()), globalTimeout)
    await click(link({class:'new'}))
    await click('New document')

    // Create random file name because NC20 will complain if file already
    // exists
    const file_rand = 'test-' + Math.random().toString(16).substr(2, 12)
    await press([...file_rand, 'Enter'])

    await waitFor(5000)

    let italicButtonId = '#id-toolbar-btn-italic'
    await waitFor(async () => (await $(italicButtonId).isVisible()), globalTimeout)

    // Activate italic button
    let buttonStateBefore = await evaluate($(italicButtonId), (elem) => {return elem.getAttribute('class')})
    await assert.ok(!buttonStateBefore.includes('active'))

    await waitFor(async () => (await $('#id_target_cursor').isVisible()), globalTimeout)
    await click($(italicButtonId))

    let buttonStateAfter = await evaluate($(italicButtonId), (elem) => {return elem.getAttribute('class')})
    await assert.ok(buttonStateAfter.includes('active'))

    await press([...'Hi from taiko!', 'Enter'])

    // Deactivate italic finially
    await click($(italicButtonId))
    await closeTab()
  } catch (error) {
    await console.error(error)
    console.log('Got an error here')
    await screenshot()
    process.exitCode = 1
  } finally {
    await closeBrowser()
  }
})()
